# Generated by Django 4.2.10 on 2024-04-23 17:35

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("database", "0035_processlock"),
    ]

    operations = [
        migrations.DeleteModel(
            name="OfflineChatProcessorConversationConfig",
        ),
    ]
